"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ToolbarButtonController_1 = require("../ToolbarButtonController");
const createViewHandle_1 = require("../utils/createViewHandle");
/**
 * Allows creating and managing toolbar buttons.
 *
 * [View the demo plugin](https://github.com/laurent22/joplin/tree/dev/packages/app-cli/tests/support/plugins/register_command)
 */
class JoplinViewsToolbarButtons {
    constructor(plugin, store) {
        this.store = store;
        this.plugin = plugin;
    }
    /**
     * Creates a new toolbar button and associate it with the given command.
     */
    create(id, commandName, location) {
        return __awaiter(this, arguments, void 0, function* () {
            if (arguments.length < 3) {
                this.plugin.deprecationNotice('1.5', 'Creating a view without an ID is deprecated. To fix it, change your call to `joplin.views.toolbarButtons.create("my-unique-id", ...)`', true);
                location = commandName;
                commandName = id;
                id = `${this.plugin.viewCount}`;
            }
            const handle = (0, createViewHandle_1.default)(this.plugin, id);
            const controller = new ToolbarButtonController_1.default(handle, this.plugin.id, this.store, commandName, location);
            this.plugin.addViewController(controller);
        });
    }
}
exports.default = JoplinViewsToolbarButtons;
//# sourceMappingURL=JoplinViewsToolbarButtons.js.map