"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Note_1 = require("../../models/Note");
const test_utils_1 = require("../../testing/test-utils");
const ShareService_1 = require("./ShareService");
const reducer_1 = require("../../reducer");
const redux_1 = require("redux");
function mockApi() {
    return {
        exec: (method, path = '', _query = null, _body = null, _headers = null, _options = null) => {
            if (method === 'GET' && path === 'api/shares')
                return { items: [] };
            return null;
        },
        personalizedUserContentBaseUrl(_userId) {
        },
    };
}
function mockService() {
    const service = new ShareService_1.default();
    const store = (0, redux_1.createStore)(reducer_1.default);
    service.initialize(store, mockApi());
    return service;
}
describe('ShareService', function () {
    beforeEach((done) => __awaiter(this, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
        done();
    }));
    it('should not change the note user timestamps when sharing or unsharing', (() => __awaiter(this, void 0, void 0, function* () {
        let note = yield Note_1.default.save({});
        const service = mockService();
        yield (0, test_utils_1.msleep)(1);
        yield service.shareNote(note.id);
        function checkTimestamps(previousNote, newNote) {
            // After sharing or unsharing, only the updated_time property should
            // be updated, for sync purposes. All other timestamps shouldn't
            // change.
            expect(previousNote.user_created_time).toBe(newNote.user_created_time);
            expect(previousNote.user_updated_time).toBe(newNote.user_updated_time);
            expect(previousNote.updated_time < newNote.updated_time).toBe(true);
            expect(previousNote.created_time).toBe(newNote.created_time);
        }
        {
            const noteReloaded = yield Note_1.default.load(note.id);
            checkTimestamps(note, noteReloaded);
            note = noteReloaded;
        }
        yield (0, test_utils_1.msleep)(1);
        yield service.unshareNote(note.id);
        {
            const noteReloaded = yield Note_1.default.load(note.id);
            checkTimestamps(note, noteReloaded);
        }
    })));
});
//# sourceMappingURL=ShareService.test.js.map