"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.masterKeyById = exports.setPpk = exports.addMasterKey = exports.masterKeyEnabled = exports.setMasterKeyEnabled = exports.getActiveMasterKey = exports.setActiveMasterKeyId = exports.getActiveMasterKeyId = exports.setEncryptionEnabled = exports.getEncryptionEnabled = exports.SyncInfo = exports.syncInfoEquals = exports.mergeSyncInfos = exports.localSyncInfoFromState = exports.localSyncInfo = exports.saveLocalSyncInfo = exports.fetchSyncInfo = exports.uploadSyncInfo = exports.migrateLocalSyncInfo = void 0;
const Setting_1 = require("../../models/Setting");
function migrateLocalSyncInfo(db) {
    return __awaiter(this, void 0, void 0, function* () {
        if (Setting_1.default.value('syncInfoCache'))
            return; // Already initialized
        // TODO: if the sync info is changed, there should be steps to migrate from
        // v3 to v4, v4 to v5, etc.
        const masterKeys = yield db.selectAll('SELECT * FROM master_keys');
        const masterKeyMap = {};
        for (const mk of masterKeys)
            masterKeyMap[mk.id] = mk;
        const syncInfo = new SyncInfo();
        syncInfo.version = Setting_1.default.value('syncVersion');
        syncInfo.e2ee = Setting_1.default.valueNoThrow('encryption.enabled', false);
        syncInfo.activeMasterKeyId = Setting_1.default.valueNoThrow('encryption.activeMasterKeyId', '');
        syncInfo.masterKeys = masterKeys;
        // We set the timestamp to 0 because we don't know when the source setting
        // has been set. That way, if the parameter is changed later on in any
        // client, the new value will have higher priority. This is to handle this
        // case:
        //
        // - Client 1 upgrade local sync target info (with E2EE = false)
        // - Client 1 set E2EE to true
        // - Client 2 upgrade local sync target info (with E2EE = false)
        // - => If we don't set the timestamp to 0, the local value of client 2 will
        //   have a higher timestamp and E2EE will get disabled, even though this is
        //   most likely not what the user wants.
        syncInfo.setKeyTimestamp('e2ee', 0);
        syncInfo.setKeyTimestamp('activeMasterKeyId', 0);
        yield saveLocalSyncInfo(syncInfo);
    });
}
exports.migrateLocalSyncInfo = migrateLocalSyncInfo;
function uploadSyncInfo(api, syncInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        yield api.put('info.json', syncInfo.serialize());
    });
}
exports.uploadSyncInfo = uploadSyncInfo;
function fetchSyncInfo(api) {
    return __awaiter(this, void 0, void 0, function* () {
        const syncTargetInfoText = yield api.get('info.json');
        // Returns version 0 if the sync target is empty
        let output = { version: 0 };
        if (syncTargetInfoText) {
            output = JSON.parse(syncTargetInfoText);
            if (!output.version)
                throw new Error('Missing "version" field in info.json');
        }
        else {
            // If info.json is not present, this might be an old sync target, in
            // which case we can at least get the version number from version.txt
            const oldVersion = yield api.get('.sync/version.txt');
            if (oldVersion)
                output = { version: 1 };
        }
        return new SyncInfo(JSON.stringify(output));
    });
}
exports.fetchSyncInfo = fetchSyncInfo;
function saveLocalSyncInfo(syncInfo) {
    Setting_1.default.setValue('syncInfoCache', syncInfo.serialize());
}
exports.saveLocalSyncInfo = saveLocalSyncInfo;
function localSyncInfo() {
    return new SyncInfo(Setting_1.default.value('syncInfoCache'));
}
exports.localSyncInfo = localSyncInfo;
function localSyncInfoFromState(state) {
    return new SyncInfo(state.settings['syncInfoCache']);
}
exports.localSyncInfoFromState = localSyncInfoFromState;
function mergeSyncInfos(s1, s2) {
    const output = new SyncInfo();
    output.setWithTimestamp(s1.keyTimestamp('e2ee') > s2.keyTimestamp('e2ee') ? s1 : s2, 'e2ee');
    output.setWithTimestamp(s1.keyTimestamp('activeMasterKeyId') > s2.keyTimestamp('activeMasterKeyId') ? s1 : s2, 'activeMasterKeyId');
    output.setWithTimestamp(s1.keyTimestamp('ppk') > s2.keyTimestamp('ppk') ? s1 : s2, 'ppk');
    output.version = s1.version > s2.version ? s1.version : s2.version;
    output.masterKeys = s1.masterKeys.slice();
    for (const mk of s2.masterKeys) {
        const idx = output.masterKeys.findIndex(m => m.id === mk.id);
        if (idx < 0) {
            output.masterKeys.push(mk);
        }
        else {
            const mk2 = output.masterKeys[idx];
            output.masterKeys[idx] = mk.updated_time > mk2.updated_time ? mk : mk2;
        }
    }
    return output;
}
exports.mergeSyncInfos = mergeSyncInfos;
function syncInfoEquals(s1, s2) {
    return s1.serialize() === s2.serialize();
}
exports.syncInfoEquals = syncInfoEquals;
class SyncInfo {
    constructor(serialized = null) {
        this.version_ = 0;
        this.masterKeys_ = [];
        this.e2ee_ = { value: false, updatedTime: 0 };
        this.activeMasterKeyId_ = { value: '', updatedTime: 0 };
        this.ppk_ = { value: null, updatedTime: 0 };
        if (serialized)
            this.load(serialized);
    }
    toObject() {
        return {
            version: this.version,
            e2ee: this.e2ee_,
            activeMasterKeyId: this.activeMasterKeyId_,
            masterKeys: this.masterKeys,
            ppk: this.ppk_,
        };
    }
    serialize() {
        return JSON.stringify(this.toObject(), null, '\t');
    }
    load(serialized) {
        const s = JSON.parse(serialized);
        this.version = 'version' in s ? s.version : 0;
        this.e2ee_ = 'e2ee' in s ? s.e2ee : { value: false, updatedTime: 0 };
        this.activeMasterKeyId_ = 'activeMasterKeyId' in s ? s.activeMasterKeyId : { value: '', updatedTime: 0 };
        this.masterKeys_ = 'masterKeys' in s ? s.masterKeys : [];
        this.ppk_ = 'ppk' in s ? s.ppk : { value: null, updatedTime: 0 };
    }
    setWithTimestamp(fromSyncInfo, propName) {
        if (!(propName in this))
            throw new Error(`Invalid prop name: ${propName}`);
        this[propName] = fromSyncInfo[propName];
        this.setKeyTimestamp(propName, fromSyncInfo.keyTimestamp(propName));
    }
    get version() {
        return this.version_;
    }
    set version(v) {
        if (v === this.version_)
            return;
        this.version_ = v;
    }
    get ppk() {
        return this.ppk_.value;
    }
    set ppk(v) {
        if (v === this.ppk_.value)
            return;
        this.ppk_ = { value: v, updatedTime: Date.now() };
    }
    get e2ee() {
        return this.e2ee_.value;
    }
    set e2ee(v) {
        if (v === this.e2ee)
            return;
        this.e2ee_ = { value: v, updatedTime: Date.now() };
    }
    get activeMasterKeyId() {
        return this.activeMasterKeyId_.value;
    }
    set activeMasterKeyId(v) {
        if (v === this.activeMasterKeyId)
            return;
        this.activeMasterKeyId_ = { value: v, updatedTime: Date.now() };
    }
    get masterKeys() {
        return this.masterKeys_;
    }
    set masterKeys(v) {
        if (JSON.stringify(v) === JSON.stringify(this.masterKeys_))
            return;
        this.masterKeys_ = v;
    }
    keyTimestamp(name) {
        if (!(`${name}_` in this))
            throw new Error(`Invalid name: ${name}`);
        return this[`${name}_`].updatedTime;
    }
    setKeyTimestamp(name, timestamp) {
        if (!(`${name}_` in this))
            throw new Error(`Invalid name: ${name}`);
        this[`${name}_`].updatedTime = timestamp;
    }
}
exports.SyncInfo = SyncInfo;
// ---------------------------------------------------------
// Shortcuts to simplify the refactoring
// ---------------------------------------------------------
function getEncryptionEnabled() {
    return localSyncInfo().e2ee;
}
exports.getEncryptionEnabled = getEncryptionEnabled;
function setEncryptionEnabled(v, activeMasterKeyId = '') {
    const s = localSyncInfo();
    s.e2ee = v;
    if (activeMasterKeyId)
        s.activeMasterKeyId = activeMasterKeyId;
    saveLocalSyncInfo(s);
}
exports.setEncryptionEnabled = setEncryptionEnabled;
function getActiveMasterKeyId() {
    return localSyncInfo().activeMasterKeyId;
}
exports.getActiveMasterKeyId = getActiveMasterKeyId;
function setActiveMasterKeyId(id) {
    const s = localSyncInfo();
    s.activeMasterKeyId = id;
    saveLocalSyncInfo(s);
}
exports.setActiveMasterKeyId = setActiveMasterKeyId;
function getActiveMasterKey(s = null) {
    s = s || localSyncInfo();
    if (!s.activeMasterKeyId)
        return null;
    return s.masterKeys.find(mk => mk.id === s.activeMasterKeyId);
}
exports.getActiveMasterKey = getActiveMasterKey;
function setMasterKeyEnabled(mkId, enabled = true) {
    const s = localSyncInfo();
    const idx = s.masterKeys.findIndex(mk => mk.id === mkId);
    if (idx < 0)
        throw new Error(`No such master key: ${mkId}`);
    // Disabled for now as it's needed to disable even the main master key when the password has been forgotten
    // https://discourse.joplinapp.org/t/syncing-error-with-joplin-cloud-and-e2ee-master-key-is-not-loaded/20115/5?u=laurent
    //
    // if (mkId === getActiveMasterKeyId() && !enabled) throw new Error('The active master key cannot be disabled');
    s.masterKeys[idx] = Object.assign(Object.assign({}, s.masterKeys[idx]), { enabled: enabled ? 1 : 0, updated_time: Date.now() });
    saveLocalSyncInfo(s);
}
exports.setMasterKeyEnabled = setMasterKeyEnabled;
function masterKeyEnabled(mk) {
    if ('enabled' in mk)
        return !!mk.enabled;
    return true;
}
exports.masterKeyEnabled = masterKeyEnabled;
function addMasterKey(syncInfo, masterKey) {
    // Sanity check - because shouldn't happen
    if (syncInfo.masterKeys.find(mk => mk.id === masterKey.id))
        throw new Error('Master key is already present');
    syncInfo.masterKeys.push(masterKey);
    saveLocalSyncInfo(syncInfo);
}
exports.addMasterKey = addMasterKey;
function setPpk(ppk) {
    const syncInfo = localSyncInfo();
    syncInfo.ppk = ppk;
    saveLocalSyncInfo(syncInfo);
}
exports.setPpk = setPpk;
function masterKeyById(id) {
    return localSyncInfo().masterKeys.find(mk => mk.id === id);
}
exports.masterKeyById = masterKeyById;
//# sourceMappingURL=syncInfoUtils.js.map