"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const htmlUtils_1 = require("../../htmlUtils");
const utils_1 = require("../../utils");
function plugin(markdownIt, ruleOptions) {
    const defaultRender = markdownIt.renderer.rules.image;
    markdownIt.renderer.rules.image = (tokens, idx, options, env, self) => {
        const Resource = ruleOptions.ResourceModel;
        const token = tokens[idx];
        const src = utils_1.default.getAttr(token.attrs, 'src');
        const title = utils_1.default.getAttr(token.attrs, 'title');
        if (!Resource.isResourceUrl(src) || ruleOptions.plainResourceRendering)
            return defaultRender(tokens, idx, options, env, self);
        const r = utils_1.default.imageReplacement(ruleOptions.ResourceModel, src, ruleOptions.resources, ruleOptions.resourceBaseUrl, ruleOptions.itemIdToUrl);
        if (typeof r === 'string')
            return r;
        if (r) {
            let js = '';
            if (ruleOptions.enableLongPress) {
                const id = r['data-resource-id'];
                const longPressHandler = `${ruleOptions.postMessageSyntax}('longclick:${id}')`;
                // if t is set when ontouchstart is called it means the user has already touched the screen once and this is the 2nd touch
                // in this case we assume the user is trying to zoom and we don't want to show the menu
                const touchStart = `if (typeof(t) !== 'undefined' && !!t) { clearTimeout(t); t = null; } else { t = setTimeout(() => { t = null; ${longPressHandler}; }, ${utils_1.default.longPressDelay}); }`;
                const cancel = 'if (!!t) clearTimeout(t); t=null';
                js = ` ontouchstart="${touchStart}" ontouchend="${cancel}" ontouchcancel="${cancel}" ontouchmove="${cancel}"`;
            }
            return `<img data-from-md ${htmlUtils_1.default.attributesHtml(Object.assign({}, r, { title: title }))}${js}/>`;
        }
        return defaultRender(tokens, idx, options, env, self);
    };
}
exports.default = { plugin };
//# sourceMappingURL=image.js.map